#include <libcbd.h>

#include <cbdutil.h>

static void
pblk_write(int fd, size_t pblk, size_t count, const u8* data)
{
    off_t off;
    size_t remain;
    ssize_t ret;

    off = lseek(fd, pblk * PBLK_SIZE, SEEK_SET);
    if (off == (off_t)-1) {
        error("Failed to seek\n");
    }

    remain = count * PBLK_SIZE;
    while (remain) {
        ret = write(fd, data, count * PBLK_SIZE);
        if (ret <= 0) {
            error("Failed to write\n");
        }
        remain -= ret;
        data += ret;
    }
}

int
cbd_format(const char* dev,
           uint64_t poff, uint64_t psize,
           uint64_t lsize, uint16_t lshift)
{
    int devfd;
    struct cbd_header header;
    uint8_t pblkbuf[PBLK_SIZE];
    uint64_t pblk;
    uint64_t zone_idx;

    devfd = open(dev, O_RDWR);
    if (devfd < 0) {
        error("Cannot open device\n");
    }

    if (!psize) {
        off_t len;
        len = lseek(devfd, 0, SEEK_END);
        if (len == (off_t)-1) {
            error("Cannot seek device\n");
        }
        psize = len >> PBLK_SHIFT;
    }
    if (!lsize) {
        lsize = psize * CBD_DEFAULT_COMPRESSION_FACTOR;
    }
    if (!lshift) {
        lshift = CBD_DEFAULT_LOGICAL_BLOCK_SHIFT;
    }

    printf("%s: paramaters...\n", __func__);
    printf("  poff=%lu psize=%lu\n", (unsigned long)poff, (unsigned long)psize);
    printf("  lsize=%lu lshift=%hu\n", (unsigned long)lsize, (unsigned short)lshift);
    memset(&header, 0, sizeof(header));
    memcpy(header.magic, CBD_MAGIC, sizeof(header.magic));
    header.version_major = CBD_VERSION_MAJOR;
    header.version_minor = CBD_VERSION_MINOR;
    header.params.algorithm = CBD_ALG_LZ4;
    header.params.compression = 1;
    header.params.lblk_shift = lshift;
    header.params.nr_pblk = psize;
    /* XXX: Initial estimate */
    header.params.lblk_per_zone = zone_data_len(&header.params) * (lsize >> lshift) / psize;
    printf("  initial estimate for lblk_per_zone: %lu\n", (unsigned long)header.params.lblk_per_zone);
    header.params.nr_zones = (psize - CBD_HEADER_BLOCKS) / zone_len(&header.params);
    header.params.lblk_per_zone = DIV_ROUND_UP((lsize >> lshift), header.params.nr_zones);
    printf("%s: parameters...\n", __func__);
    printf("  lblk_shift=%hu\n", (unsigned short)header.params.lblk_shift);
    printf("  nr_pblk=%lu\n", (unsigned long)header.params.nr_pblk);
    printf("  nr_zones=%lu\n", (unsigned long)header.params.nr_zones);
    printf("  lblk_per_zone=%lu\n", (unsigned long)header.params.lblk_per_zone);

    memset(pblkbuf, 0, sizeof(pblkbuf));
    cbd_header_put(pblkbuf, &header);

    pblk = poff;
    pblk_write(devfd, pblk, 1, pblkbuf);
    pblk += CBD_HEADER_BLOCKS;

    printf("Writing %lu zones starting at %lu\n",
            (unsigned long)header.params.nr_zones, (unsigned long)pblk);

    memset(pblkbuf, 0, sizeof(pblkbuf));
    for (zone_idx = 0; zone_idx < header.params.nr_zones; ++zone_idx) {
        uint32_t count;
        uint32_t n;
        pblk = zone_off(&header.params, zone_idx);
        count = zone_metadata_len(&header.params);
        printf("  Zone %lu pblk=%lu count=%u\n",
                (unsigned long)zone_idx, (unsigned long)pblk, (unsigned int)count);
        for (n = 0; n < count; ++n) {
            pblk_write(devfd, pblk + n, 1, pblkbuf);
        }
    }

    return 0;
}
